@@ -1,68 +1,63 @@
-#!/usr/bin/env python
-
-__author__ = 'rolandh'
-
 import copy
-import sys
-import os
-import re
+import importlib
 import logging
 import logging.handlers
+import os
+import re
+import sys
+from logging.config import dictConfig as configure_logging_by_dict
+from warnings import warn as _warn
+
 import six
 
-from importlib import import_module
-
-from saml2 import root_logger, BINDING_URI, SAMLError
-from saml2 import BINDING_SOAP
-from saml2 import BINDING_HTTP_REDIRECT
-from saml2 import BINDING_HTTP_POST
 from saml2 import BINDING_HTTP_ARTIFACT
+from saml2 import BINDING_HTTP_POST
+from saml2 import BINDING_HTTP_REDIRECT
+from saml2 import BINDING_SOAP
+from saml2 import BINDING_URI
+from saml2 import SAMLError
 
 from saml2.attribute_converter import ac_factory
 from saml2.assertion import Policy
 from saml2.mdstore import MetadataStore
+from saml2.saml import NAME_FORMAT_URI
 from saml2.virtual_org import VirtualOrg
 
+
 logger = logging.getLogger(__name__)
 
-from saml2 import md
-from saml2 import saml
-from saml2.extension import mdui
-from saml2.extension import idpdisc
-from saml2.extension import dri
-from saml2.extension import mdattr
-from saml2.extension import ui
-from saml2 import xmldsig
-from saml2 import xmlenc
+__author__ = 'rolandh'
 
 
-ONTS = {
-    saml.NAMESPACE: saml,
-    mdui.NAMESPACE: mdui,
-    mdattr.NAMESPACE: mdattr,
-    dri.NAMESPACE: dri,
-    ui.NAMESPACE: ui,
-    idpdisc.NAMESPACE: idpdisc,
-    md.NAMESPACE: md,
-    xmldsig.NAMESPACE: xmldsig,
-    xmlenc.NAMESPACE: xmlenc
-}
-
 COMMON_ARGS = [
-    "entityid", "xmlsec_binary", "debug", "key_file", "cert_file", "encryption_keypairs", "additional_cert_files",
-    "metadata_key_usage", "secret", "accepted_time_diff", "name", "ca_certs",
-    "description", "valid_for", "verify_ssl_cert",
+    "logging",
+    "debug",
+    "entityid",
+    "xmlsec_binary",
+    "key_file",
+    "cert_file",
+    "encryption_keypairs",
+    "additional_cert_files",
+    "metadata_key_usage",
+    "secret",
+    "accepted_time_diff",
+    "name",
+    "ca_certs",
+    "description",
+    "valid_for",
+    "verify_ssl_cert",
     "organization",
     "contact_person",
     "name_form",
     "virtual_organization",
-    "logger",
     "only_use_keys_in_metadata",
-    "logout_requests_signed",
     "disable_ssl_certificate_validation",
-    "referred_binding",
+    "preferred_binding",
     "session_storage",
+    "assurance_certification",
+    "entity_attributes",
     "entity_category",
+    "entity_category_support",
     "xmlsec_path",
     "extension_schemas",
     "cert_handler_extra_class",
@@ -74,7 +69,15 @@ COMMON_ARGS = [
     "tmp_key_file",
     "validate_certificate",
     "extensions",
-    "allow_unknown_attributes"
+    "allow_unknown_attributes",
+    "crypto_backend",
+    "delete_tmpfiles",
+    "endpoints",
+    "metadata",
+    "ui_info",
+    "name_id_format",
+    "signing_algorithm",
+    "digest_algorithm",
 ]
 
 SP_ARGS = [
@@ -85,14 +88,22 @@ SP_ARGS = [
     "subject_data",
     "want_response_signed",
     "want_assertions_signed",
+    "want_assertions_or_response_signed",
     "authn_requests_signed",
     "name_form",
-    "endpoints",
-    "ui_info",
     "discovery_response",
     "allow_unsolicited",
     "ecp",
-    "name_id_format",
+    "name_id_policy_format",
+    "name_id_format_allow_create",
+    "logout_requests_signed",
+    "logout_responses_signed",
+    "requested_attribute_name_format",
+    "hide_assertion_consumer_service",
+    "force_authn",
+    "sp_type",
+    "sp_type_in_metadata",
+    "requested_attributes",
 ]
 
 AA_IDP_ARGS = [
@@ -107,10 +118,6 @@ AA_IDP_ARGS = [
     "subject_data",
     "sp",
     "scope",
-    "endpoints",
-    "metadata",
-    "ui_info",
-    "name_id_format",
     "domain",
     "name_qualifier",
     "edu_person_targeted_id",
@@ -123,40 +130,20 @@ AQ_ARGS = ["endpoints"]
 AA_ARGS = ["attribute", "attribute_profile"]
 
 COMPLEX_ARGS = ["attribute_converters", "metadata", "policy"]
-ALL = set(COMMON_ARGS + SP_ARGS + AA_IDP_ARGS + PDP_ARGS + COMPLEX_ARGS +
-          AA_ARGS)
+ALL = set(COMMON_ARGS + SP_ARGS + AA_IDP_ARGS + PDP_ARGS + COMPLEX_ARGS + AA_ARGS)
 
 SPEC = {
-    "": COMMON_ARGS + COMPLEX_ARGS,
-    "sp": COMMON_ARGS + COMPLEX_ARGS + SP_ARGS,
+    "":    COMMON_ARGS + COMPLEX_ARGS,
+    "sp":  COMMON_ARGS + COMPLEX_ARGS + SP_ARGS,
     "idp": COMMON_ARGS + COMPLEX_ARGS + AA_IDP_ARGS,
-    "aa": COMMON_ARGS + COMPLEX_ARGS + AA_IDP_ARGS + AA_ARGS,
+    "aa":  COMMON_ARGS + COMPLEX_ARGS + AA_IDP_ARGS + AA_ARGS,
     "pdp": COMMON_ARGS + COMPLEX_ARGS + PDP_ARGS,
-    "aq": COMMON_ARGS + COMPLEX_ARGS + AQ_ARGS,
+    "aq":  COMMON_ARGS + COMPLEX_ARGS + AQ_ARGS,
 }
 
-# --------------- Logging stuff ---------------
-
-LOG_LEVEL = {
-    'debug': logging.DEBUG,
-    'info': logging.INFO,
-    'warning': logging.WARNING,
-    'error': logging.ERROR,
-    'critical': logging.CRITICAL}
-
-LOG_HANDLER = {
-    "rotating": logging.handlers.RotatingFileHandler,
-    "syslog": logging.handlers.SysLogHandler,
-    "timerotate": logging.handlers.TimedRotatingFileHandler,
-    "memory": logging.handlers.MemoryHandler,
-}
-
-LOG_FORMAT = "%(asctime)s %(name)s:%(levelname)s %(message)s"
-
 _RPA = [BINDING_HTTP_REDIRECT, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT]
 _PRA = [BINDING_HTTP_POST, BINDING_HTTP_REDIRECT, BINDING_HTTP_ARTIFACT]
-_SRPA = [BINDING_SOAP, BINDING_HTTP_REDIRECT, BINDING_HTTP_POST,
-         BINDING_HTTP_ARTIFACT]
+_SRPA = [BINDING_SOAP, BINDING_HTTP_REDIRECT, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT]
 
 PREFERRED_BINDING = {
     "single_logout_service": _SRPA,
@@ -176,13 +163,12 @@ PREFERRED_BINDING = {
 class ConfigurationError(SAMLError):
     pass
 
-# -----------------------------------------------------------------
-
 
 class Config(object):
     def_context = ""
 
     def __init__(self, homedir="."):
+        self.logging = None
         self._homedir = homedir
         self.entityid = None
         self.xmlsec_binary = None
@@ -204,10 +190,12 @@ class Config(object):
         self.contact_person = None
         self.name_form = None
         self.name_id_format = None
+        self.name_id_policy_format = None
+        self.name_id_format_allow_create = None
         self.virtual_organization = None
-        self.logger = None
         self.only_use_keys_in_metadata = True
         self.logout_requests_signed = None
+        self.logout_responses_signed = None
         self.disable_ssl_certificate_validation = None
         self.context = ""
         self.attribute_converters = None
@@ -218,11 +206,13 @@ class Config(object):
         self.preferred_binding = PREFERRED_BINDING
         self.domain = ""
         self.name_qualifier = ""
-        self.entity_category = ""
+        self.assurance_certification = []
+        self.entity_attributes = []
+        self.entity_category = []
+        self.entity_category_support = []
         self.crypto_backend = 'xmlsec1'
         self.scope = ""
         self.allow_unknown_attributes = False
-        self.allow_unsolicited = False
         self.extension_schema = {}
         self.cert_handler_extra_class = None
         self.verify_encrypt_cert_advice = None
@@ -235,6 +225,10 @@ class Config(object):
         self.extensions = {}
         self.attribute = []
         self.attribute_profile = []
+        self.requested_attribute_name_format = NAME_FORMAT_URI
+        self.delete_tmpfiles = True
+        self.signing_algorithm = None
+        self.digest_algorithm = None
 
     def setattr(self, context, attr, val):
         if context == "":
@@ -251,79 +245,53 @@ class Config(object):
         else:
             return getattr(self, "_%s_%s" % (context, attr), None)
 
-    def load_special(self, cnf, typ, metadata_construction=False):
+    def load_special(self, cnf, typ):
         for arg in SPEC[typ]:
             try:
-                self.setattr(typ, arg, cnf[arg])
+                _val = cnf[arg]
             except KeyError:
                 pass
+            else:
+                if _val == "true":
+                    _val = True
+                elif _val == "false":
+                    _val = False
+                self.setattr(typ, arg, _val)
 
         self.context = typ
-        self.load_complex(cnf, typ, metadata_construction=metadata_construction)
         self.context = self.def_context
 
-    def load_complex(self, cnf, typ="", metadata_construction=False):
+    def load_complex(self, cnf):
+        acs = ac_factory(cnf.get("attribute_map_dir"))
+        if not acs:
+            raise ConfigurationError("No attribute converters, something is wrong!!")
+        self.setattr("", "attribute_converters", acs)
+
         try:
-            self.setattr(typ, "policy", Policy(cnf["policy"]))
+            self.setattr("", "metadata", self.load_metadata(cnf["metadata"]))
         except KeyError:
             pass
 
-        # for srv, spec in cnf["service"].items():
-        #     try:
-        #         self.setattr(srv, "policy",
-        #                      Policy(cnf["service"][srv]["policy"]))
-        #     except KeyError:
-        #         pass
+        for srv, spec in cnf.get("service", {}).items():
+            policy_conf = spec.get("policy")
+            self.setattr(srv, "policy", Policy(policy_conf, self.metadata))
 
-        try:
-            try:
-                acs = ac_factory(cnf["attribute_map_dir"])
-            except KeyError:
-                acs = ac_factory()
-
-            if not acs:
-                raise ConfigurationError(
-                    "No attribute converters, something is wrong!!")
-
-            _acs = self.getattr("attribute_converters", typ)
-            if _acs:
-                _acs.extend(acs)
-            else:
-                self.setattr(typ, "attribute_converters", acs)
-
-        except KeyError:
-            pass
-
-        if not metadata_construction:
-            try:
-                self.setattr(typ, "metadata",
-                             self.load_metadata(cnf["metadata"]))
-            except KeyError:
-                pass
-
-    def unicode_convert(self, item):
-        try:
-            return six.text_type(item, "utf-8")
-        except TypeError:
-            _uc = self.unicode_convert
-            if isinstance(item, dict):
-                return dict([(key, _uc(val)) for key, val in item.items()])
-            elif isinstance(item, list):
-                return [_uc(v) for v in item]
-            elif isinstance(item, tuple):
-                return tuple([_uc(v) for v in item])
-            else:
-                return item
-
-    def load(self, cnf, metadata_construction=False):
+    def load(self, cnf, metadata_construction=None):
         """ The base load method, loads the configuration
 
         :param cnf: The configuration as a dictionary
-        :param metadata_construction: Is this only to be able to construct
-            metadata. If so some things can be left out.
         :return: The Configuration instance
         """
-        _uc = self.unicode_convert
+
+        if metadata_construction is not None:
+            warn_msg = (
+                "The metadata_construction parameter for saml2.config.Config.load "
+                "is deprecated and ignored; "
+                "instead, initialize the Policy object setting the mds param."
+            )
+            logger.warning(warn_msg)
+            _warn(warn_msg, DeprecationWarning)
+
         for arg in COMMON_ARGS:
             if arg == "virtual_organization":
                 if "virtual_organization" in cnf:
@@ -338,18 +306,27 @@ class Config(object):
                         self.extension_schema[_mod.NAMESPACE] = _mod
 
             try:
-                setattr(self, arg, _uc(cnf[arg]))
+                setattr(self, arg, cnf[arg])
             except KeyError:
                 pass
             except TypeError:  # Something that can't be a string
                 setattr(self, arg, cnf[arg])
 
+        if self.logging is not None:
+            configure_logging_by_dict(self.logging)
+
+        if not self.delete_tmpfiles:
+            warn_msg = (
+                "Configuration option `delete_tmpfiles` is set to False; "
+                "consider setting this to True to have temporary files deleted."
+            )
+            logger.warning(warn_msg)
+            _warn(warn_msg)
+
         if "service" in cnf:
             for typ in ["aa", "idp", "sp", "pdp", "aq"]:
                 try:
-                    self.load_special(
-                        cnf["service"][typ], typ,
-                        metadata_construction=metadata_construction)
+                    self.load_special(cnf["service"][typ], typ)
                     self.serves.append(typ)
                 except KeyError:
                     pass
@@ -357,7 +334,7 @@ class Config(object):
         if "extensions" in cnf:
             self.do_extensions(cnf["extensions"])
 
-        self.load_complex(cnf, metadata_construction=metadata_construction)
+        self.load_complex(cnf)
         self.context = self.def_context
 
         return self
@@ -370,24 +347,30 @@ class Config(object):
         else:
             sys.path.insert(0, head)
 
-        return import_module(tail)
+        return importlib.import_module(tail)
 
-    def load_file(self, config_file, metadata_construction=False):
-        if config_file.endswith(".py"):
-            config_file = config_file[:-3]
+    def load_file(self, config_filename, metadata_construction=None):
+        if metadata_construction is not None:
+            warn_msg = (
+                "The metadata_construction parameter for saml2.config.Config.load_file "
+                "is deprecated and ignored; "
+                "instead, initialize the Policy object setting the mds param."
+            )
+            logger.warning(warn_msg)
+            _warn(warn_msg, DeprecationWarning)
 
-        mod = self._load(config_file)
-        #return self.load(eval(open(config_file).read()))
-        return self.load(copy.deepcopy(mod.CONFIG), metadata_construction)
+        if config_filename.endswith(".py"):
+            config_filename = config_filename[:-3]
+
+        mod = self._load(config_filename)
+        return self.load(copy.deepcopy(mod.CONFIG))
 
     def load_metadata(self, metadata_conf):
         """ Loads metadata into an internal structure """
 
         acs = self.attribute_converters
-
         if acs is None:
-            raise ConfigurationError(
-                "Missing attribute converter specification")
+            raise ConfigurationError("Missing attribute converter specification")
 
         try:
             ca_certs = self.ca_certs
@@ -398,8 +381,7 @@ class Config(object):
         except:
             disable_validation = False
 
-        mds = MetadataStore(
-            list(ONTS.values()), acs, self, ca_certs,
+        mds = MetadataStore(acs, self, ca_certs,
             disable_ssl_certificate_validation=disable_validation)
 
         mds.imp(metadata_conf)
@@ -422,7 +404,12 @@ class Config(object):
         if endps and service in endps:
             for endpspec in endps[service]:
                 try:
-                    endp, bind = endpspec
+                    # endspec sometime is str, sometime is a tuple
+                    if type(endpspec) in (tuple, list):
+                        # slice prevents 3-tuple, eg: sp's assertion_consumer_service
+                        endp, bind = endpspec[0:2]
+                    else:
+                        endp, bind = endpspec
                     if binding is None or bind == binding:
                         spec.append(endp)
                 except ValueError:
@@ -433,63 +420,6 @@ class Config(object):
         else:
             return unspec
 
-    def log_handler(self):
-        try:
-            _logconf = self.logger
-        except KeyError:
-            return None
-
-        handler = None
-        for htyp in LOG_HANDLER:
-            if htyp in _logconf:
-                if htyp == "syslog":
-                    args = _logconf[htyp]
-                    if "socktype" in args:
-                        import socket
-                        if args["socktype"] == "dgram":
-                            args["socktype"] = socket.SOCK_DGRAM
-                        elif args["socktype"] == "stream":
-                            args["socktype"] = socket.SOCK_STREAM
-                        else:
-                            raise ConfigurationError("Unknown socktype!")
-                    try:
-                        handler = LOG_HANDLER[htyp](**args)
-                    except TypeError:  # difference between 2.6 and 2.7
-                        del args["socktype"]
-                        handler = LOG_HANDLER[htyp](**args)
-                else:
-                    handler = LOG_HANDLER[htyp](**_logconf[htyp])
-                break
-
-        if handler is None:
-            # default if rotating logger
-            handler = LOG_HANDLER["rotating"]()
-
-        if "format" in _logconf:
-            formatter = logging.Formatter(_logconf["format"])
-        else:
-            formatter = logging.Formatter(LOG_FORMAT)
-
-        handler.setFormatter(formatter)
-        return handler
-
-    def setup_logger(self):
-        if root_logger.level != logging.NOTSET:  # Someone got there before me
-            return root_logger
-
-        _logconf = self.logger
-        if _logconf is None:
-            return root_logger
-
-        try:
-            root_logger.setLevel(LOG_LEVEL[_logconf["loglevel"].lower()])
-        except KeyError:  # reasonable default
-            root_logger.setLevel(logging.INFO)
-
-        root_logger.addHandler(self.log_handler())
-        root_logger.info("Logging started")
-        return root_logger
-
     def endpoint2service(self, endpoint, context=None):
         endps = self.getattr("endpoints", context)
 
@@ -504,6 +434,22 @@ class Config(object):
         for key, val in extensions.items():
             self.extensions[key] = val
 
+    def service_per_endpoint(self, context=None):
+        """
+        List all endpoint this entity publishes and which service and binding
+        that are behind the endpoint
+
+        :param context: Type of entity
+        :return: Dictionary with endpoint url as key and a tuple of
+            service and binding as value
+        """
+        endps = self.getattr("endpoints", context)
+        res = {}
+        for service, specs in endps.items():
+            for endp, binding in specs:
+                res[endp] = (service, binding)
+        return res
+
 
 class SPConfig(Config):
     def_context = "sp"
@@ -540,14 +486,30 @@ class IdPConfig(Config):
         Config.__init__(self)
 
 
-def config_factory(typ, filename):
-    if typ == "sp":
-        conf = SPConfig().load_file(filename)
-        conf.context = typ
-    elif typ in ["aa", "idp", "pdp", "aq"]:
-        conf = IdPConfig().load_file(filename)
-        conf.context = typ
+def config_factory(_type, config):
+    """
+
+    :type _type: str
+    :param _type:
+
+    :type config: str or dict
+    :param config: Name of file with pysaml2 config or CONFIG dict
+
+    :return:
+    """
+    if _type == "sp":
+        conf = SPConfig()
+    elif _type in ["aa", "idp", "pdp", "aq"]:
+        conf = IdPConfig()
     else:
-        conf = Config().load_file(filename)
-        conf.context = typ
+        conf = Config()
+
+    if isinstance(config, dict):
+        conf.load(copy.deepcopy(config))
+    elif isinstance(config, str):
+        conf.load_file(config)
+    else:
+        raise ValueError('Unknown type of config')
+
+    conf.context = _type
     return conf
