@@ -33,6 +33,12 @@ logger = logging.getLogger(__name__)
 
 
 class Client(Entity):
+    """ECP-aware client that works on the client (application) side.
+
+    You can use this class when you want to login user through
+    ECP-aware SP and IdP.
+    """
+
     def __init__(self, user, passwd, sp="", idp=None, metadata_file=None,
                  xmlsec_binary=None, verbose=0, ca_certs="",
                  disable_ssl_certificate_validation=True, key_file=None,
@@ -74,7 +80,7 @@ class Client(Entity):
         if metadata_file:
             self._metadata = MetadataStore([saml, samlp], None, config)
             self._metadata.load("local", metadata_file)
-            logger.debug("Loaded metadata from '%s'" % metadata_file)
+            logger.debug("Loaded metadata from '%s'", metadata_file)
         else:
             self._metadata = None
 
@@ -85,8 +91,16 @@ class Client(Entity):
         self.done_ecp = False
         self.cookie_jar = cookielib.LWPCookieJar()
 
-    def phase2(self, authn_request, rc_url, idp_entity_id, headers=None,
-               sign=False, **kwargs):
+    def phase2(
+        self,
+        authn_request,
+        rc_url,
+        idp_entity_id,
+        headers=None,
+        sign=False,
+        sign_alg=None,
+        **kwargs,
+    ):
         """
         Doing the second phase of the ECP conversation, the conversation
         with the IdP happens.
@@ -99,27 +113,30 @@ class Client(Entity):
         :return: The response from the IdP
         """
 
-        _, destination = self.pick_binding("single_sign_on_service",
-                                           [BINDING_SOAP], "idpsso",
-                                           entity_id=idp_entity_id)
+        _, destination = self.pick_binding(
+            "single_sign_on_service", [BINDING_SOAP], "idpsso", entity_id=idp_entity_id
+        )
 
-        ht_args = self.apply_binding(BINDING_SOAP, authn_request, destination,
-                                     sign=sign)
+        ht_args = self.apply_binding(
+            BINDING_SOAP, authn_request, destination, sign=sign, sigalg=sign_alg
+        )
 
         if headers:
             ht_args["headers"].extend(headers)
 
-        logger.debug("[P2] Sending request: %s" % ht_args["data"])
+        logger.debug("[P2] Sending request: %s", ht_args["data"])
 
         # POST the request to the IdP
         response = self.send(**ht_args)
 
-        logger.debug("[P2] Got IdP response: %s" % response)
+        logger.debug("[P2] Got IdP response: %s", response)
 
         if response.status_code != 200:
             raise SAMLError(
-                "Request to IdP failed (%s): %s" % (response.status_code,
-                                                    response.error))
+                "Request to IdP failed ({status}): {text}".format(
+                    status=response.status_code, text=response.text
+                )
+            )
 
         # SAMLP response in a SOAP envelope body, ecp response in headers
         respdict = self.parse_soap_message(response.text)
@@ -127,12 +144,19 @@ class Client(Entity):
         if respdict is None:
             raise SAMLError("Unexpected reply from the IdP")
 
-        logger.debug("[P2] IdP response dict: %s" % respdict)
+        logger.debug("[P2] IdP response dict: %s", respdict)
 
         idp_response = respdict["body"]
-        assert idp_response.c_tag == "Response"
 
-        logger.debug("[P2] IdP AUTHN response: %s" % idp_response)
+        expected_tag = "Response"
+        if idp_response.c_tag != expected_tag:
+            raise ValueError(
+                "Invalid Response tag '{invalid}' should be '{valid}'".format(
+                    invalid=idp_response.c_tag, valid=expected_tag
+                )
+            )
+
+        logger.debug("[P2] IdP AUTHN response: %s", idp_response)
 
         _ecp_response = None
         for item in respdict["header"]:
@@ -155,11 +179,18 @@ class Client(Entity):
         if respdict is None:
             raise SAMLError("Unexpected reply from the SP")
 
-        logger.debug("[P1] SP response dict: %s" % respdict)
+        logger.debug("[P1] SP response dict: %s", respdict)
 
         # AuthnRequest in the body or not
         authn_request = respdict["body"]
-        assert authn_request.c_tag == "AuthnRequest"
+
+        expected_tag = "AuthnRequest"
+        if authn_request.c_tag != expected_tag:
+            raise ValueError(
+                "Invalid AuthnRequest tag '{invalid}' should be '{valid}'".format(
+                    invalid=authn_request.c_tag, valid=expected_tag
+                )
+            )
 
         # ecp.RelayState among headers
         _relay_state = None
@@ -175,8 +206,11 @@ class Client(Entity):
 
         _rc_url = _paos_request.response_consumer_url
 
-        return {"authn_request": authn_request, "rc_url": _rc_url,
-                "relay_state": _relay_state}
+        return {
+            "authn_request": authn_request,
+            "rc_url": _rc_url,
+            "relay_state": _relay_state,
+        }
 
     def ecp_conversation(self, respdict, idp_entity_id=None):
         """
@@ -198,33 +232,29 @@ class Client(Entity):
         # Phase 3 - back to the SP
         # **********************************
 
-        ht_args = self.use_soap(idp_response, args["rc_url"],
-                                [args["relay_state"]])
-
-        logger.debug("[P3] Post to SP: %s" % ht_args["data"])
-
-        ht_args["headers"].append(('Content-Type', 'application/vnd.paos+xml'))
+        ht_args = self.use_soap(idp_response, args["rc_url"], [args["relay_state"]])
+        ht_args["headers"][0] = ('Content-Type', MIME_PAOS)
+        logger.debug("[P3] Post to SP: %s", ht_args["data"])
 
         # POST the package from the IdP to the SP
-        response = self.send(args["rc_url"], "POST", **ht_args)
+        response = self.send(**ht_args)
 
         if response.status_code == 302:
             # ignore where the SP is redirecting us to and go for the
             # url I started off with.
             pass
         else:
-            print(response.error)
-            raise SAMLError(
-                "Error POSTing package to SP: %s" % response.error)
+            raise SAMLError("Error POSTing package to SP: %s" % response.text)
 
-        logger.debug("[P3] SP response: %s" % response.text)
+        logger.debug("[P3] SP response: %s", response.text)
 
         self.done_ecp = True
         logger.debug("Done ECP")
 
         return None
 
-    def add_paos_headers(self, headers=None):
+    @staticmethod
+    def add_paos_headers(headers=None):
         if headers:
             headers = set_list2dict(headers)
             headers["PAOS"] = PAOS_HEADER_INFO
@@ -255,8 +285,7 @@ class Client(Entity):
         :param opargs: Arguments to the HTTP call
         :return: The page
         """
-        if url not in opargs:
-            url = self._sp
+        sp_url = self._sp
 
         # ********************************************
         # Phase 1 - First conversation with the SP
@@ -264,13 +293,13 @@ class Client(Entity):
         # headers needed to indicate to the SP that I'm ECP enabled
 
         opargs["headers"] = self.add_paos_headers(opargs["headers"])
-
-        response = self.send(url, op, **opargs)
+        response = self.send(sp_url, op, **opargs)
         logger.debug("[Op] SP response: %s" % response)
+        print(response.text)
 
         if response.status_code != 200:
             raise SAMLError(
-                "Request to SP failed: %s" % response.error)
+                "Request to SP failed: %s" % response.text)
 
         # The response might be a AuthnRequest instance in a SOAP envelope
         # body. If so it's the start of the ECP conversation
@@ -282,19 +311,16 @@ class Client(Entity):
         # header blocks may also be present
         try:
             respdict = self.parse_soap_message(response.text)
-
             self.ecp_conversation(respdict, idp_entity_id)
 
             # should by now be authenticated so this should go smoothly
             response = self.send(url, op, **opargs)
         except (soap.XmlParseError, AssertionError, KeyError):
-            pass
+            raise
 
-        #print("RESP",response, self.http.response)
-
-        if  response.status_code != 404:
+        if response.status_code >= 400:
             raise SAMLError("Error performing operation: %s" % (
-                response.error,))
+                response.text,))
 
         return response
 
