@@ -3,6 +3,19 @@
 #
 # Generated Mon May  2 14:23:33 2011 by parse_xsd.py version 0.4.
 #
+# A summary of available specifications can be found at:
+# https://wiki.oasis-open.org/security/FrontPage
+#
+# saml core specifications to be found at:
+# if any question arise please query the following pdf.
+# http://docs.oasis-open.org/security/saml/v2.0/saml-core-2.0-os.pdf
+# The specification was later updated with errata, and the new version is here:
+# https://www.oasis-open.org/committees/download.php/56776/sstc-saml-core-errata-2.0-wd-07.pdf
+#
+
+
+import base64
+
 from saml2.validate import valid_ipv4, MustValueError
 from saml2.validate import valid_ipv6
 from saml2.validate import ShouldValueError
@@ -15,28 +28,53 @@ import six
 from saml2 import xmldsig as ds
 from saml2 import xmlenc as xenc
 
+# authentication information fields
 NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:assertion'
 
-XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
+# xmlschema definition
+XSD = "xs"
+# xmlschema templates and extensions
 XS_NAMESPACE = 'http://www.w3.org/2001/XMLSchema'
-
+# xmlschema-instance, which contains several builtin attributes
+XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
+# xml soap namespace
+NS_SOAP_ENC = "http://schemas.xmlsoap.org/soap/encoding/"
+# type definitions for xmlschemas
 XSI_TYPE = '{%s}type' % XSI_NAMESPACE
+# nil type definition for xmlschemas 
 XSI_NIL = '{%s}nil' % XSI_NAMESPACE
 
+# idp and sp communicate usually about a subject(NameID)
+# the format determines the category the subject is in
+
+# custom subject
+NAMEID_FORMAT_UNSPECIFIED = (
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified")
+# subject as email address
 NAMEID_FORMAT_EMAILADDRESS = (
     "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress")
-NAMEID_FORMAT_UNSPECIFIED1 = (
-    "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified")
-NAMEID_FORMAT_UNSPECIFIED = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:unspecified")
-NAMEID_FORMAT_ENCRYPTED = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:encrypted")
-NAMEID_FORMAT_PERSISTENT = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent")
-NAMEID_FORMAT_TRANSIENT = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:transient")
+# subject as x509 key
+NAMEID_FORMAT_X509SUBJECTNAME = (
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:X509SubjectName")
+# subject as windows domain name
+NAMEID_FORMAT_WINDOWSDOMAINQUALIFIEDNAME = (
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:WindowsDomainQualifiedName")
+# subject from a kerberos instance
+NAMEID_FORMAT_KERBEROS = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:kerberos")
+# subject as name
 NAMEID_FORMAT_ENTITY = (
     "urn:oasis:names:tc:SAML:2.0:nameid-format:entity")
+# linked subject
+NAMEID_FORMAT_PERSISTENT = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent")
+# annonymous subject
+NAMEID_FORMAT_TRANSIENT = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:transient")
+# subject avaiable in encrypted format
+NAMEID_FORMAT_ENCRYPTED = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:encrypted")
+# dicc for avaiable formats
 NAMEID_FORMATS_SAML2 = (
     ('NAMEID_FORMAT_EMAILADDRESS', NAMEID_FORMAT_EMAILADDRESS),
     ('NAMEID_FORMAT_ENCRYPTED', NAMEID_FORMAT_ENCRYPTED),
@@ -45,82 +83,80 @@ NAMEID_FORMATS_SAML2 = (
     ('NAMEID_FORMAT_TRANSIENT', NAMEID_FORMAT_TRANSIENT),
     ('NAMEID_FORMAT_UNSPECIFIED', NAMEID_FORMAT_UNSPECIFIED),
 )
+
+# a profile outlines a set of rules describing how to embed SAML assertions.
+# https://docs.oasis-open.org/security/saml/v2.0/saml-profiles-2.0-os.pdf
+# The specification was later updated with errata, and the new version is here:
+# https://www.oasis-open.org/committees/download.php/56782/sstc-saml-profiles-errata-2.0-wd-07.pdf
+
+# XML based values for SAML attributes  
 PROFILE_ATTRIBUTE_BASIC = (
     "urn:oasis:names:tc:SAML:2.0:profiles:attribute:basic")
 
+# an AuthnRequest is made to initiate authentication
+# authenticate the request with login credentials
 AUTHN_PASSWORD = "urn:oasis:names:tc:SAML:2.0:ac:classes:Password"
+# authenticate the request with login credentials, over tls/https
 AUTHN_PASSWORD_PROTECTED = \
     "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport"
 
+# attribute statements is key:value metadata shared with your app
+
+# custom format
 NAME_FORMAT_UNSPECIFIED = (
     "urn:oasis:names:tc:SAML:2.0:attrname-format:unspecified")
+# uri format
 NAME_FORMAT_URI = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri"
+# XML-based format
 NAME_FORMAT_BASIC = "urn:oasis:names:tc:SAML:2.0:attrname-format:basic"
+# dicc for avaiable formats
 NAME_FORMATS_SAML2 = (
     ('NAME_FORMAT_BASIC', NAME_FORMAT_BASIC),
     ('NAME_FORMAT_URI', NAME_FORMAT_URI),
     ('NAME_FORMAT_UNSPECIFIED', NAME_FORMAT_UNSPECIFIED),
 )
+
+# the SAML authority's decision can be predetermined by arbitrary context
+
+# the specified action is permitted
 DECISION_TYPE_PERMIT = "Permit"
+# the specified action is denied
 DECISION_TYPE_DENY = "Deny"
+# the SAML authority cannot determine if the action is permitted or denied
 DECISION_TYPE_INDETERMINATE = "Indeterminate"
 
+
+# consent attributes determine wether consent has been given and under
+# what conditions
+
+# no claim to consent is made
 CONSENT_UNSPECIFIED = "urn:oasis:names:tc:SAML:2.0:consent:unspecified"
+# consent has been obtained
 CONSENT_OBTAINED = "urn:oasis:names:tc:SAML:2.0:consent:obtained"
+# consent has been obtained before the message has been initiated
 CONSENT_PRIOR = "urn:oasis:names:tc:SAML:2.0:consent:prior"
+# consent has been obtained implicitly
 CONSENT_IMPLICIT = "urn:oasis:names:tc:SAML:2.0:consent:current-implicit"
+# consent has been obtained explicitly
 CONSENT_EXPLICIT = "urn:oasis:names:tc:SAML:2.0:consent:current-explicit"
+# no consent has been obtained
 CONSENT_UNAVAILABLE = "urn:oasis:names:tc:SAML:2.0:consent:unavailable"
+# no consent is needed.
 CONSENT_INAPPLICABLE = "urn:oasis:names:tc:SAML:2.0:consent:inapplicable"
 
+
+# Subject confirmation methods(scm), can be issued, besides the subject itself
+# by third parties.
+# http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.0.pdf
+
+# the 3rd party is identified on behalf of the subject given private/public key
 SCM_HOLDER_OF_KEY = "urn:oasis:names:tc:SAML:2.0:cm:holder-of-key"
+# the 3rd party is identified by subject confirmation and must include a security header
+# signing its content.
 SCM_SENDER_VOUCHES = "urn:oasis:names:tc:SAML:2.0:cm:sender-vouches"
+# a bearer token is issued instead.
 SCM_BEARER = "urn:oasis:names:tc:SAML:2.0:cm:bearer"
 
-# -----------------------------------------------------------------------------
-XSD = "xs:"
-NS_SOAP_ENC = "http://schemas.xmlsoap.org/soap/encoding/"
-
-# -----------------------------------------------------------------------------
-
-
-def _decode_attribute_value(typ, text):
-    if typ == XSD + "string":
-        return text or ""
-    if typ == XSD + "integer" or typ == XSD + "int":
-        return str(int(text))
-    if typ == XSD + "float" or typ == XSD + "double":
-        return str(float(text))
-    if typ == XSD + "boolean":
-        return "%s" % (text == "true" or text == "True")
-    if typ == XSD + "base64Binary":
-        import base64
-
-        return base64.decodestring(text)
-    raise ValueError("type %s not supported" % type)
-
-
-def _verify_value_type(typ, val):
-    #print("verify value type: %s, %s" % (typ, val))
-    if typ == XSD + "string":
-        try:
-            return str(val)
-        except UnicodeEncodeError:
-            return unicode(val)
-    if typ == XSD + "integer" or typ == XSD + "int":
-        return int(val)
-    if typ == XSD + "float" or typ == XSD + "double":
-        return float(val)
-    if typ == XSD + "boolean":
-        if val.lower() == "true" or val.lower() == "false":
-            pass
-        else:
-            raise ValueError("Faulty boolean value")
-    if typ == XSD + "base64Binary":
-        import base64
-
-        return base64.decodestring(val.encode('utf-8'))
-
 
 class AttributeValueBase(SamlBase):
     def __init__(self,
@@ -136,10 +172,12 @@ class AttributeValueBase(SamlBase):
         if self._extatt:
             self.extension_attributes = self._extatt
 
-        if not text:
-            self.extension_attributes = {XSI_NIL: 'true'}
-        else:
+        if text:
             self.set_text(text)
+        elif not extension_elements:
+            self.extension_attributes = {XSI_NIL: 'true'}
+        elif XSI_TYPE in self.extension_attributes:
+            del self.extension_attributes[XSI_TYPE]
 
     def __setattr__(self, key, value):
         if key == "text":
@@ -148,9 +186,15 @@ class AttributeValueBase(SamlBase):
             SamlBase.__setattr__(self, key, value)
 
     def verify(self):
-        if not self.text:
-            assert self.extension_attributes
-            assert self.extension_attributes[XSI_NIL] == "true"
+        if not self.text and not self.extension_elements:
+            if not self.extension_attributes:
+                raise Exception(
+                    "Attribute value base should not have extension attributes"
+                )
+            if self.extension_attributes[XSI_NIL] != "true":
+                raise Exception(
+                    "Attribute value base should not have extension attributes"
+                )
             return True
         else:
             SamlBase.verify(self)
@@ -158,7 +202,7 @@ class AttributeValueBase(SamlBase):
     def set_type(self, typ):
         try:
             del self.extension_attributes[XSI_NIL]
-        except KeyError:
+        except (AttributeError, KeyError):
             pass
 
         try:
@@ -166,6 +210,18 @@ class AttributeValueBase(SamlBase):
         except AttributeError:
             self._extatt[XSI_TYPE] = typ
 
+        if typ.startswith('xs:'):
+            try:
+                self.extension_attributes['xmlns:xs'] = XS_NAMESPACE
+            except AttributeError:
+                self._extatt['xmlns:xs'] = XS_NAMESPACE
+
+        if typ.startswith('xsd:'):
+            try:
+                self.extension_attributes['xmlns:xsd'] = XS_NAMESPACE
+            except AttributeError:
+                self._extatt['xmlns:xsd'] = XS_NAMESPACE
+
     def get_type(self):
         try:
             return self.extension_attributes[XSI_TYPE]
@@ -185,68 +241,127 @@ class AttributeValueBase(SamlBase):
         except KeyError:
             pass
 
-    def set_text(self, val, base64encode=False):
-        typ = self.get_type()
-        if base64encode:
-            import base64
+    def set_text(self, value, base64encode=False):
+        def _wrong_type_value(xsd, value):
+            msg = 'Type and value do not match: {xsd}:{type}:{value}'
+            msg = msg.format(xsd=xsd, type=type(value), value=value)
+            raise ValueError(msg)
 
-            val = base64.encodestring(val)
-            self.set_type("xs:base64Binary")
-        else:
-            if isinstance(val, six.binary_type):
-                val = val.decode('utf-8')
-            if isinstance(val, six.string_types):
-                if not typ:
-                    self.set_type("xs:string")
-                else:
-                    try:
-                        assert typ == "xs:string"
-                    except AssertionError:
-                        if typ == "xs:int":
-                            _ = int(val)
-                        elif typ == "xs:boolean":
-                            if val.lower() not in ["true", "false"]:
-                                raise ValueError("Not a boolean")
-                        elif typ == "xs:float":
-                            _ = float(val)
-                        elif typ == "xs:base64Binary":
-                            pass
-                        else:
-                            ValueError("Type and value doesn't match")
-            elif isinstance(val, bool):
-                if val:
-                    val = "true"
-                else:
-                    val = "false"
-                if not typ:
-                    self.set_type("xs:boolean")
-                else:
-                    assert typ == "xs:boolean"
-            elif isinstance(val, int):
-                val = str(val)
-                if not typ:
-                    self.set_type("xs:integer")
-                else:
-                    assert typ == "xs:integer"
-            elif isinstance(val, float):
-                val = str(val)
-                if not typ:
-                    self.set_type("xs:float")
-                else:
-                    assert typ == "xs:float"
-            elif not val:
-                try:
-                    self.extension_attributes[XSI_TYPE] = typ
-                except AttributeError:
-                    self._extatt[XSI_TYPE] = typ
-                val = ""
-            else:
-                if typ == "xs:anyType":
-                    pass
-                else:
-                    raise ValueError
+        # only work with six.string_types
+        if isinstance(value, six.binary_type):
+            value = value.decode('utf-8')
 
-        SamlBase.__setattr__(self, "text", val)
+        type_to_xsd = {
+            str:        'string',
+            int:        'integer',
+            float:      'float',
+            bool:       'boolean',
+            type(None): '',
+        }
+
+        # entries of xsd-types each declaring:
+        # - a corresponding python type
+        # - a function to turn a string into that type
+        # - a function to turn that type into a text-value
+        xsd_types_props = {
+            'string': {
+                'type': str,
+                'to_type': str,
+                'to_text': str,
+            },
+            'integer': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'short': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'int': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'long': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'float': {
+                'type': float,
+                'to_type': float,
+                'to_text': str,
+            },
+            'double': {
+                'type': float,
+                'to_type': float,
+                'to_text': str,
+            },
+            'boolean': {
+                'type': bool,
+                'to_type': lambda x: {
+                    'true': True,
+                    'false': False,
+                }[str(x).lower()],
+                'to_text': lambda x: str(x).lower(),
+            },
+            'base64Binary': {
+                'type': str,
+                'to_type': str,
+                'to_text': (
+                    lambda x: base64.encodebytes(x.encode()) if base64encode else x
+                ),
+            },
+            'anyType': {
+                'type': type(value),
+                'to_type': lambda x: x,
+                'to_text': lambda x: x,
+            },
+            '': {
+                'type': type(None),
+                'to_type': lambda x: None,
+                'to_text': lambda x: '',
+            },
+        }
+
+        xsd_string = (
+            'base64Binary' if base64encode
+            else self.get_type()
+            or type_to_xsd.get(type(value)))
+
+        xsd_ns, xsd_type = (
+            ['', type(None)] if xsd_string is None
+            else ['', ''] if xsd_string == ''
+            else [
+                XSD if xsd_string in xsd_types_props else '',
+                xsd_string
+            ] if ':' not in xsd_string
+            else xsd_string.split(':', 1))
+
+        xsd_type_props = xsd_types_props.get(xsd_type, {})
+        valid_type = xsd_type_props.get('type', type(None))
+        to_type = xsd_type_props.get('to_type', str)
+        to_text = xsd_type_props.get('to_text', str)
+
+        # cast to correct type before type-checking
+        if type(value) is str and valid_type is not str:
+            try:
+                value = to_type(value)
+            except (TypeError, ValueError, KeyError) as e:
+                # the cast failed
+                _wrong_type_value(xsd=xsd_type, value=value)
+
+        if type(value) is not valid_type:
+            _wrong_type_value(xsd=xsd_type, value=value)
+
+        text = to_text(value)
+        self.set_type(
+            '{ns}:{type}'.format(ns=xsd_ns, type=xsd_type) if xsd_ns
+            else xsd_type if xsd_type
+            else '')
+        SamlBase.__setattr__(self, 'text', text)
         return self
 
     def harvest_element_tree(self, tree):
@@ -255,18 +370,28 @@ class AttributeValueBase(SamlBase):
             self._convert_element_tree_to_member(child)
         for attribute, value in iter(tree.attrib.items()):
             self._convert_element_attribute_to_member(attribute, value)
-        if tree.text:
+
+        # if we have added children to this node
+        # we consider whitespace insignificant
+        # and remove/trim/strip whitespace
+        # and expect to not have actual text content
+        text = (
+            tree.text.strip()
+            if tree.text and self.extension_elements
+            else tree.text
+        )
+        if text:
             #print("set_text:", tree.text)
             # clear type
             #self.clear_type()
-            self.set_text(tree.text)
+            self.set_text(text)
+
+        # if we have added a text node
+        # or other children to this node
+        # remove the nil marker
+        if text or self.extension_elements:
             if XSI_NIL in self.extension_attributes:
                 del self.extension_attributes[XSI_NIL]
-            try:
-                typ = self.extension_attributes[XSI_TYPE]
-                _verify_value_type(typ, getattr(self, "text"))
-            except KeyError:
-                pass
 
 
 class BaseIDAbstractType_(SamlBase):
@@ -441,7 +566,7 @@ class SubjectConfirmationDataType_(SamlBase):
     c_attributes = SamlBase.c_attributes.copy()
     c_child_order = SamlBase.c_child_order[:]
     c_cardinality = SamlBase.c_cardinality.copy()
-    c_attributes['NotBefore'] = ('not_before', 'AsTime', False)
+    c_attributes['NotBefore'] = ('not_before', 'dateTime', False)
     c_attributes['NotOnOrAfter'] = ('not_on_or_after', 'dateTime', False)
     c_attributes['Recipient'] = ('recipient', 'anyURI', False)
     c_attributes['InResponseTo'] = ('in_response_to', 'NCName', False)
@@ -799,7 +924,16 @@ def base_id_from_string(xml_string):
 
 
 class NameID(NameIDType_):
-    """The urn:oasis:names:tc:SAML:2.0:assertion:NameID element """
+    """The urn:oasis:names:tc:SAML:2.0:assertion:NameID element
+
+    From the Oasis SAML2 Technical Overview:
+
+    "The <NameID> element within a <Subject> offers the ability to provide name
+     identifiers in a number of different formats. SAML's predefined formats
+     include: Email address, X.509 subject name, Windows domain qualified name,
+     Kerberos principal name, Entity identifier, Persistent identifier,
+     Transient identifier."
+     """
 
     c_tag = 'NameID'
     c_namespace = NAMESPACE
@@ -993,12 +1127,11 @@ class AuthnContextType_(SamlBase):
         self.authenticating_authority = authenticating_authority or []
 
     def verify(self):
-        # either <AuthnContextDecl> or <AuthnContextDeclRef> not both
-        if self.authn_context_decl:
-            assert self.authn_context_decl_ref is None
-        elif self.authn_context_decl_ref:
-            assert self.authn_context_decl is None
-
+        if self.authn_context_decl and self.authn_context_decl_ref:
+            raise Exception(
+                "Invalid Response: "
+                "Cannot have both <AuthnContextDecl> and <AuthnContextDeclRef>"
+            )
         return SamlBase.verify(self)
 
 
@@ -1057,6 +1190,11 @@ class AttributeType_(SamlBase):
         self.name_format = name_format
         self.friendly_name = friendly_name
 
+    # when consuming such elements, default to NAME_FORMAT_UNSPECIFIED as NameFormat
+    def harvest_element_tree(self, tree):
+        tree.attrib.setdefault('NameFormat', NAME_FORMAT_UNSPECIFIED)
+        SamlBase.harvest_element_tree(self, tree)
+
 
 def attribute_type__from_string(xml_string):
     return saml2.create_class_from_xml_string(AttributeType_, xml_string)
@@ -1228,9 +1366,11 @@ class ConditionsType_(SamlBase):
 
     def verify(self):
         if self.one_time_use:
-            assert len(self.one_time_use) == 1
+            if len(self.one_time_use) != 1:
+                raise Exception("Cannot be used more than once")
         if self.proxy_restriction:
-            assert len(self.proxy_restriction) == 1
+            if len(self.proxy_restriction) != 1:
+                raise Exception("Cannot be used more than once")
 
         return SamlBase.verify(self)
 
