@@ -14,6 +14,21 @@ CONFIG = {
             "required_attributes": ["surName", "givenName", "mail"],
             "optional_attributes": ["title"],
             "idp": ["urn:mace:example.com:saml:roland:idp"],
+            "logout_responses_signed": True,
+            "logout_requests_signed": True,
+            "requested_attributes": [
+                {
+                    "name": "urn:oid:1.3.6.1.4.1.5923.1.1.1.2",
+                    "required": False,
+                },
+                {
+                    "friendly_name": "eduPersonNickname",
+                    "required": True,
+                },
+                {
+                    "friendly_name": "eduPersonScopedAffiliation",
+                },
+            ],
         }
     },
     "debug": 1,
