@@ -1,30 +1,10 @@
-__author__ = 'rolandh'
-
 from saml2 import md
 from saml2.mdie import from_dict
+from saml2.mdstore import load_metadata_modules
 
-from saml2 import saml
-
-from saml2.extension import mdui
-from saml2.extension import idpdisc
-from saml2.extension import dri
-from saml2.extension import mdattr
-from saml2.extension import ui
-from saml2 import xmldsig
-from saml2 import xmlenc
-
-ONTS = {
-    saml.NAMESPACE: saml,
-    mdui.NAMESPACE: mdui,
-    mdattr.NAMESPACE: mdattr,
-    dri.NAMESPACE: dri,
-    ui.NAMESPACE: ui,
-    idpdisc.NAMESPACE: idpdisc,
-    md.NAMESPACE: md,
-    xmldsig.NAMESPACE: xmldsig,
-    xmlenc.NAMESPACE: xmlenc
-}
+__author__ = 'rolandh'
 
+ONTS = load_metadata_modules()
 
 def _eq(l1, l2):
     return set(l1) == set(l2)
