@@ -1,13 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import base64
+from base64 import encodebytes as b64encode
+
+import pytest
 from saml2.authn_context import INTERNETPROTOCOLPASSWORD
 from saml2.saml import NAMEID_FORMAT_TRANSIENT
 from saml2.samlp import NameIDPolicy
-from saml2.s2repoze.plugins.sp import make_plugin
 from saml2.server import Server
 
+try:
+    from saml2.s2repoze.plugins.sp import make_plugin
+except ImportError:
+    make_plugin = None
+
 ENV1 = {'SERVER_SOFTWARE': 'CherryPy/3.1.2 WSGI Server',
         'SCRIPT_NAME': '',
         'ACTUAL_SERVER_PROTOCOL': 'HTTP/1.1',
@@ -43,9 +49,13 @@ AUTHN = {
 }
 
 
+@pytest.mark.skipif(not make_plugin,
+                    reason="s2repoze dependencies not installed")
 class TestSP():
     def setup_class(self):
         self.sp = make_plugin("rem", saml_conf="server_conf")
+        # Explicitly allow unsigned responses for this test
+        self.sp.saml_client.want_response_signed = False
         self.server = Server(config_file="idp_conf")
 
     def teardown_class(self):
@@ -64,7 +74,7 @@ class TestSP():
             "urn:mace:example.com:saml:roland:sp", trans_name_policy,
             "foba0001@example.com", authn=AUTHN)
 
-        resp_str = base64.encodestring(resp_str.encode('utf-8'))
+        resp_str = b64encode(resp_str.encode('utf-8'))
         self.sp.outstanding_queries = {"id1": "http://www.example.com/service"}
         session_info = self.sp._eval_authn_response(
             {}, {"SAMLResponse": [resp_str]})
