@@ -2,20 +2,8 @@
 
 import argparse
 
-from saml2 import saml
-from saml2 import md
-from saml2 import xmldsig
-from saml2 import xmlenc
-
 from saml2.attribute_converter import ac_factory
 from saml2.httpbase import HTTPBase
-from saml2.extension import dri
-from saml2.extension import idpdisc
-from saml2.extension import mdattr
-from saml2.extension import mdrpi
-from saml2.extension import mdui
-from saml2.extension import shibmd
-from saml2.extension import ui
 
 from saml2.sigver import _get_xmlsec_cryptobackend
 from saml2.sigver import SecurityContext
